#include "viewer.h"

int main(int argc, char** argv) {
  GtkWidget * window;   
  GtkWidget * vbox;
  GtkWidget * frame;
  GtkWidget * bbox;
  GtkWidget * rec_button;
  GtkWidget * clr_button;
  GtkWidget * led_button;
  GtkWidget * conv_button;
  GtkWidget * halt_button;
  GtkWidget * filt_button;

  //initialize terminal device
  device_fd = open( MODEMDEVICE, O_RDWR | O_NOCTTY ); 
  if(device_fd < 0) { 
    printf("file open failed " MODEMDEVICE ": %s\n", strerror(errno));
    return 1; 
  }
  tcgetattr(device_fd, &device_oldtio);
  memset(&device_newtio, '\0', sizeof(device_newtio));
  device_newtio.c_cflag = BAUDRATE | CS8 | CLOCAL | CREAD;
  device_newtio.c_iflag = IGNPAR;
  device_newtio.c_oflag = 0;
  device_newtio.c_lflag     = 0;         
  device_newtio.c_cc[VTIME] = 0;    //inter-character timer x * 0.1s 
  device_newtio.c_cc[VMIN]  = 0;    //blocking read until x chars received 
  tcsetattr(device_fd,TCSANOW, &device_newtio);
  tcflush(device_fd, TCIFLUSH);

  //initialize sample buffer
  for( samplewpos = 0; samplewpos < SAMPLEBUFSIZE; samplewpos++ ) {
   sample[samplewpos] = SAMPLEFILLVAL;
  }
  samplewpos = 0;

  //initialize output file
  outfile = fopen(RECORDFILE,"w");
  if( outfile == NULL ) {
    printf("file open failed: " RECORDFILE);
    return 1;
  }

  //initialize gtk
  gtk_init(&argc, &argv);
  
  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title(GTK_WINDOW(window), "Signal Viewer");
  gtk_container_set_border_width(GTK_CONTAINER(window), 3);
  
  vbox = gtk_vbox_new(FALSE, 0);
  gtk_container_add(GTK_CONTAINER(window), vbox);
  gtk_widget_show(vbox);
  
  frame = gtk_frame_new(NULL);
  gtk_box_pack_start(GTK_BOX(vbox), frame, TRUE, TRUE, 0);
  gtk_widget_show(frame);

  graph = gtk_drawing_area_new();
  gtk_drawing_area_size(GTK_DRAWING_AREA(graph), 600, 200);
  gtk_container_add(GTK_CONTAINER(frame), graph);
  gtk_widget_show(graph);
 
  statusbar = gtk_statusbar_new();
  context = gtk_statusbar_get_context_id(GTK_STATUSBAR(statusbar), "status");
  gtk_box_pack_end(GTK_BOX(vbox), statusbar, FALSE, FALSE, 0);
  gtk_widget_show(statusbar);

  bbox = gtk_hbox_new(TRUE, 0);
  gtk_box_pack_start(GTK_BOX(vbox), bbox, FALSE, FALSE, 0);
  gtk_widget_show(bbox);

  filt_button = gtk_button_new_with_label("FILTER");
  g_signal_connect(G_OBJECT(filt_button), "button-press-event", G_CALLBACK(filt_button_press_event), NULL);
  gtk_box_pack_start(GTK_BOX(bbox), filt_button, TRUE, TRUE, 0);
  gtk_widget_show(filt_button);

  rec_button = gtk_button_new_with_label("RECORD");
  g_signal_connect(G_OBJECT(rec_button), "button-press-event", G_CALLBACK(rec_button_press_event), NULL);
  gtk_box_pack_start(GTK_BOX(bbox), rec_button, TRUE, TRUE, 0);
  gtk_widget_show(rec_button);

  clr_button = gtk_button_new_with_label("CLEAR");
  g_signal_connect(G_OBJECT(clr_button), "button-press-event", G_CALLBACK(clr_button_press_event), NULL);
  gtk_box_pack_start(GTK_BOX(bbox), clr_button, TRUE, TRUE, 0);
  gtk_widget_show(clr_button);

  led_button = gtk_button_new_with_label("LED");
  g_signal_connect(G_OBJECT(led_button), "button-press-event", G_CALLBACK(led_button_press_event), NULL);
  gtk_box_pack_start(GTK_BOX(bbox), led_button, TRUE, TRUE, 0);
  gtk_widget_show(led_button);  

  conv_button = gtk_button_new_with_label("CONV");
  g_signal_connect(G_OBJECT(conv_button), "button-press-event", G_CALLBACK(conv_button_press_event), NULL);
  gtk_box_pack_start(GTK_BOX(bbox), conv_button, TRUE, TRUE, 0);
  gtk_widget_show(conv_button);  

  halt_button = gtk_button_new_with_label("HALT");
  g_signal_connect(G_OBJECT(halt_button), "button-press-event", G_CALLBACK(halt_button_press_event), NULL);
  gtk_box_pack_start(GTK_BOX(bbox), halt_button, TRUE, TRUE, 0);
  gtk_widget_show(halt_button);  

  g_signal_connect(G_OBJECT(window), "delete-event", G_CALLBACK(window_delete_event), NULL);
  g_signal_connect(G_OBJECT(graph), "expose-event", G_CALLBACK(graph_expose_event), NULL);
  g_timeout_add(33, (GSourceFunc)graph_update, NULL);

  gtk_widget_show(window);
  gtk_main();
 
  fclose(outfile);

  tcflush(device_fd, TCIFLUSH);
  tcsetattr(device_fd,TCSANOW,&device_oldtio);
  close(device_fd);

  return(0);
}

void poll_data() {
  gint nb = read(device_fd, (guint8*)sample + samplewposb, SAMPLEBUFSIZEB - samplewposb);
  if(nb < 0) { printf("read failed: %s\n", strerror(errno)); gtk_main_quit(); } 
  samplewposb += nb;
  samplewposb %= SAMPLEBUFSIZEB;
  samplewpos = samplewposb / sizeof(guint16);
}

void status(const gchar * text) {
  if(mstackp - mstack > 99) { return; }
  *(mstackp++) = gtk_statusbar_push(GTK_STATUSBAR(statusbar), context, text);
  g_timeout_add_seconds(2, &unstatus, NULL);
}

gboolean unstatus(gpointer data) {
  gtk_statusbar_remove(GTK_STATUSBAR(statusbar), context, *(--mstackp));
  return FALSE;
}

void draw_flat() {
  gdk_draw_rectangle(graph->window,
       graph->style->bg_gc[GTK_WIDGET_STATE(graph)], TRUE, 
       0, 0, graph->allocation.width, graph->allocation.height);
} 

void draw_grid() {
  //The width width of the graph will always be
  //6s (major lines), each broken into 
  //0.2s intervals (minor lines). Eventually
  //the horizontal lines will indicate voltage.
  cairo_t * c;
  double dash = 1.5;
  guint width = graph->allocation.width;
  guint height = graph->allocation.height;
  double minorw = (double)width / 30;
  guint w; 
  c = gdk_cairo_create(graph->window);
  cairo_set_line_width(c, 0.5);
  cairo_move_to(c, 0, (double)height / 2);
  cairo_line_to(c, (double)width, (double)height / 2);
  cairo_stroke(c); 
  for(w = 1; w <= 29; w++) {
    if(w % 5 == 0) { cairo_set_dash(c, &dash, 0, 0); }
    else { cairo_set_dash(c, &dash, 1, 0); }
    cairo_move_to(c, w * minorw, 0);
    cairo_line_to(c, w * minorw, height);
    cairo_stroke(c);
  }
  cairo_destroy(c);
}

void draw_data() {
  cairo_t * c;
  double intervalh = graph->allocation.height / 1023.0;
  double intervalw = graph->allocation.width / 6000.0;
  c = gdk_cairo_create(graph->window);
  cairo_set_line_width(c, 0.5);
  cairo_set_line_cap(c, CAIRO_LINE_CAP_ROUND); 
  cairo_move_to(c, graphx*intervalw, graphy*intervalh);
  while ( samplerpos != samplewpos ) {
    if( flags & FLAG_FILTER ) { 
      sample[samplerpos]  = ALPHA * sample[samplerpos];
      sample[samplerpos] += (1-ALPHA) * sample[samplerpos == 0 ? SAMPLEBUFSIZE - 1 : samplerpos - 1];
    } 
    if( flags & FLAG_RECORD ) { fprintf(outfile, "%u\n", sample[samplerpos]); }
    cairo_line_to(c, samplerpos*intervalw,sample[samplerpos]*intervalh);
    samplerpos++;
    samplerpos %= SAMPLEBUFSIZE;
  }
  cairo_stroke(c);
  cairo_destroy(c);
  graphx = samplerpos - 1 >= 0 ? samplerpos - 1 : 0;
  graphy = sample[samplerpos - 1 >= 0 ? samplerpos - 1 : 0];
}

gboolean graph_update(gpointer * data) {
  poll_data();
  draw_data();
  return TRUE;
}

gboolean filt_button_press_event(GtkWidget * widget, GdkEventButton * event, gpointer data) {
  if( flags & FLAG_FILTER ) { status("filtering stopped"); }
  else { status("filtering"); }
  flags ^= FLAG_FILTER;
  return TRUE;
}

gboolean rec_button_press_event(GtkWidget * widget, GdkEventButton * event, gpointer data) {
  if( flags & FLAG_RECORD ) { status("recording stopped"); }
  else { status("recording"); }
  flags ^= FLAG_RECORD;
  return TRUE;
}

gboolean clr_button_press_event(GtkWidget * widget, GdkEventButton * event, gpointer data) {   
  draw_flat();
  draw_grid();
  return TRUE;
}

gboolean led_button_press_event(GtkWidget * widget, GdkEventButton * event, gpointer data) {   
  if(write(device_fd, "L", 1) < 1) { status("write failed"); }
  return TRUE;
}

gboolean conv_button_press_event(GtkWidget * widget, GdkEventButton * event, gpointer data) {   
  if(write(device_fd, "C", 1) < 1) { status("write failed"); }
  return TRUE;
}

gboolean halt_button_press_event(GtkWidget * widget, GdkEventButton * event, gpointer data) {   
  if(write(device_fd, "H", 1) < 1) { status("write failed"); }
  return TRUE;
}

gboolean window_delete_event(GtkWidget * widget, GdkEvent * event, gpointer data) {
  gtk_main_quit();
  return TRUE;
}

gboolean graph_expose_event(GtkWidget * widget, GdkEventExpose * event, gpointer data) {
  draw_grid();
  return TRUE;
}

