#include <gtk/gtk.h>
#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <errno.h>
#include <unistd.h>
#include <termios.h>
#include <features.h>
#include <sys/types.h>

//low-pass filter
#define PI 3.14159265358979323846
#define FILTFREQ 15.0
#define SAMPFREQ 1000.0
#define ALPHA    1.0/(1.0+SAMPFREQ*(1.0/PI/FILTFREQ/2.0))

#define BAUDRATE B57600
#define MODEMDEVICE "/dev/ttyUSB0"
#define RECORDFILE  "recording.dat"
#define SAMPLEMAXVAL  0x03FF
#define SAMPLEFILLVAL 0x01FF
#define SAMPLEBUFSIZE 6000
#define SAMPLEBUFSIZEB (SAMPLEBUFSIZE<<1)
//global variables
GtkWidget      * graph;                 //graph widget (GtkDrawingArea)
gint             graphx = 0;            //current x graph position
gint             graphy = 1023/2;       //current y graph position
GtkWidget      * statusbar;             //status widget (GtkStatusbar)
gint             mstack[100];           //status message id stack
gint           * mstackp = mstack;      //status message id stack pointer
guint            context;               //status context
guint16          sample[SAMPLEBUFSIZE]; //sample for 6s @ 1000 samples/s 
gint             samplewposb = 0;       //sample write position (guint8)
gint             samplewpos = 0;        //sample write position (guint16)
gint             samplerpos = 0;        //sample read position (guint16)
int              device_fd = -1;        //device file descriptor
struct termios   device_oldtio;         //device old tio struct
struct termios   device_newtio;         //device new tio struct
FILE           * outfile;               //FILE pointer to write data
gint32           flags;                 //status flags

//flags
#define FLAG_RECORD (1<<0)
#define FLAG_FILTER (1<<1)

//callback to unset the statusbar message
gboolean unstatus(gpointer data);

//set the statusbar message
void status(const gchar * text);

//draw the graph grid
void draw_grid();

//draw the graph points
void draw_data();

//poll the device for data
void poll_data();

//graph update timeout callback
gboolean graph_update(gpointer * data);

//button press event callbacks
gboolean  rec_button_press_event(GtkWidget * widget, GdkEventButton * event, gpointer data); 
gboolean  clr_button_press_event(GtkWidget * widget, GdkEventButton * event, gpointer data); 
gboolean  led_button_press_event(GtkWidget * widget, GdkEventButton * event, gpointer data); 
gboolean conv_button_press_event(GtkWidget * widget, GdkEventButton * event, gpointer data); 
gboolean halt_button_press_event(GtkWidget * widget, GdkEventButton * event, gpointer data); 
gboolean filt_button_press_event(GtkWidget * widget, GdkEventButton * event, gpointer data); 

//window delete event callback
gboolean window_delete_event(GtkWidget * widget, GdkEvent * event, gpointer data);

//graph expose event callback
gboolean graph_expose_event(GtkWidget * widget, GdkEventExpose * event, gpointer data);

